// *******************************************************************
// Copyright (C) 2009 Telos Systems, TLS Corp.
// All rights reserved.
//
// No part of this source code or associated documentation may be 
// reproduced or utilized in any form or by any means, electronic or 
// mechanical, without permission in writing from TLS Corp.
// -------------------------------------------------------------------
//      File: APSend.h
// Description: See class description.
//    Author: Ioan L. Rus
//      Date: Nov 18, 2009
// Revisions: 
// -------------------------------------------------------------------
// INI Date        Description
// --- ----------- ---------------------------------------------------
// 
// *******************************************************************
#include "APDecodeEvents.h"
#include "Msg2.h"
#include "IAPEvents.h"

// Set this to 1 to print the messages as
// they are decoded.
#define PRINT_DECODED_PACKETS 0

#if PRINT_DECODED_PACKETS
  #include <stdio.h>
  #define TRACE1(x) printf x
#else
  #define TRACE1(x)
#endif


bool APDecodeEvents::HandleEnumerateShowsCallback(CMsg2 &msg)
{
  const char * pszShowName=msg.GetString("SHOW");
  bool bRequiresPass      =msg.GetNumber("PASS")!=0;
  bool bIsActive          =msg.GetNumber("ACTV")!=0;
  const char * pszHostName=msg.GetString("HOST");
  bool bIsLast            =msg.GetNumber("LAST")!=0;

  TRACE1(("EnumerateShowsCallback(%s,%s,%s,%s,%s)\n",
    pszShowName,
    bRequiresPass ? "pass" : "no_pass",
    bIsActive ? "active" : "inactive",
    pszHostName,
    bIsLast ? "last" : "not_last"));

  if(m_pEvents==0) return false;
  else return m_pEvents->EnumerateShowsCallback(pszShowName,bRequiresPass,bIsActive,pszHostName,bIsLast);
}

bool APDecodeEvents::HandleEnumerateHostsCallback(CMsg2 &msg)
{
  const char * pszHostName=msg.GetString("HOST");
  bool bIsLast=msg.GetNumber("LAST")!=0;

  TRACE1(("EnumerateHostsCallback(%s,%s)\n",pszHostName,bIsLast ? "last" : "not_last"));

  if(m_pEvents==0) return false;
  else return m_pEvents->EnumerateHostsCallback(pszHostName,bIsLast);
}

bool APDecodeEvents::HandleEnumerateShowsForHost(CMsg2 &msg)
{
  const char * pszShowName=msg.GetString("SHOW");
  bool bIsLast=msg.GetNumber("LAST")!=0;

  TRACE1(("EnumerateShowsForHostCallback(%s,%s)\n",pszShowName,bIsLast ? "last" : "not_last"));

  if(m_pEvents==0) return false;
  else return m_pEvents->EnumerateShowsForHostCallback(pszShowName,bIsLast);
}

bool APDecodeEvents::HandleEnumerateDirectorsCallback(CMsg2 &msg)
{
  int nSession=msg.GetNumber("SSN_");
  const char * pszDirectorName=msg.GetString("DNAM");
  bool bIsTalent=msg.GetNumber("TLNT")!=0;
  bool bIsFree=msg.GetNumber("FREE")!=0;
  bool bIsLast=msg.GetNumber("LAST")!=0;

  TRACE1(("EnumerateDirectorsCallback(%d,%s,%s,%s,%s)\n",
    nSession,
    pszDirectorName,
    bIsTalent ? "talent" : "producer",
    bIsFree ? "free" : "not_free",
    bIsLast ? "last" : "not_last"));

  if(m_pEvents==0) return false;
  else return m_pEvents->EnumerateDirectorsCallback(nSession,pszDirectorName,bIsTalent,bIsFree,bIsLast);
}

bool APDecodeEvents::HandleSendShowInfo(CMsg2 &msg)
{
  int nSession=msg.GetNumber("SSN_");
  int nMaxLines=msg.GetNumber("NLIN");
  int nHybrids=msg.GetNumber("NHYB");

  TRACE1(("SendShowInfo(%d,%d,%d)\n",nSession,nMaxLines,nHybrids));

  if(m_pEvents==0) return false;
  else return m_pEvents->SendShowInfo(nSession,nMaxLines,nHybrids);
}

bool APDecodeEvents::HandleSendMode(CMsg2 &msg)
{
  int nSession=msg.GetNumber("SSN_");
  bool bTalent=msg.GetNumber("TLNT")!=0;

  TRACE1(("SendMode(%d,%s)\n",nSession,bTalent ? "talent" : "producer"));
  
  if(m_pEvents==0) return false;
  else return m_pEvents->SendMode(nSession,bTalent);
}

bool APDecodeEvents::HandleSendShowChange(CMsg2 &msg)
{
  int nSession=msg.GetNumber("SSN_");
  const char * pszNewShowName=msg.GetString("NEWS");
  const char * pszOldShowName=msg.GetString("OLDS");

  TRACE1(("SendShowChange(%d,new=%s,old=%s)\n",nSession,pszNewShowName,pszOldShowName));

  if(m_pEvents==0) return false;
  else return m_pEvents->SendShowChange(nSession,pszNewShowName,pszOldShowName);
}

bool APDecodeEvents::HandleSendRecord(CMsg2 &msg)
{
  int nSession=msg.GetNumber("SSN_");
  bool bOn=msg.GetNumber("ON__")!=0;

  TRACE1(("SendRecord(%d,%s)\n",nSession,bOn ? "on" : "off"));

  if(m_pEvents==0) return false;
  else return m_pEvents->SendRecord(nSession,bOn);
}

bool APDecodeEvents::HandleSendBusyAll(CMsg2 &msg)
{
  int nSession=msg.GetNumber("SSN_");
  bool bOn=msg.GetNumber("ON__")!=0;

  TRACE1(("SendBusyAll(%d,%s)\n",nSession,bOn ? "on" : "off"));

  if(m_pEvents==0) return false;
  else return m_pEvents->SendBusyAll(nSession,bOn);
}

bool APDecodeEvents::HandleSendTransfer(CMsg2 &msg)
{
  int nSession=msg.GetNumber("SSN_");
  bool bOn=msg.GetNumber("ON__")!=0;

  TRACE1(("SendTransfer(%d,%s)\n",nSession,bOn ? "on" : "off"));

  if(m_pEvents==0) return false;
  else return m_pEvents->SendTransfer(nSession,bOn);
}

bool APDecodeEvents::HandleSendMute(CMsg2 &msg)
{
  int nSession=msg.GetNumber("SSN_");
  bool bOn=msg.GetNumber("ON__")!=0;

  TRACE1(("SendMute(%d,%s)\n",nSession,bOn ? "on" : "off"));

  if(m_pEvents==0) return false;
  else return m_pEvents->SendMute(nSession,bOn);
}

bool APDecodeEvents::HandleSendRingMute(CMsg2 &msg)
{
  int nSession=msg.GetNumber("SSN_");
  bool bOn=msg.GetNumber("ON__")!=0;

  TRACE1(("SendRingMute(%d,%s)\n",nSession,bOn ? "on" : "off"));

  if(m_pEvents==0) return false;
  else return m_pEvents->SendRingMute(nSession,bOn);
}

bool APDecodeEvents::HandleSendDelayDump(CMsg2 &msg)
{
  int nSession=msg.GetNumber("SSN_");
  bool bOn=msg.GetNumber("ON__")!=0;

  TRACE1(("SendDelayDump(%d,%s)\n",nSession,bOn ? "on" : "off"));

  if(m_pEvents==0) return false;
  else return m_pEvents->SendDelayDump(nSession,bOn);
}

bool APDecodeEvents::HandleSendHandset(CMsg2 &msg)
{
  int nSession=msg.GetNumber("SSN_");
  bool bOn=msg.GetNumber("ON__")!=0;

  TRACE1(("SendHandset(%d,%s)\n",nSession,bOn ? "on" : "off"));

  if(m_pEvents==0) return false;
  else return m_pEvents->SendHandset(nSession,bOn);
}

bool APDecodeEvents::HandleSendSpeaker(CMsg2 &msg)
{
  int nSession=msg.GetNumber("SSN_");
  bool bOn=msg.GetNumber("ON__")!=0;

  TRACE1(("SendSpeaker(%d,%s)\n",nSession,bOn ? "on" : "off"));

  if(m_pEvents==0) return false;
  else return m_pEvents->SendSpeaker(nSession,bOn);
}

bool APDecodeEvents::HandleSendLineState(CMsg2 &msg)
{
  int nSession=msg.GetNumber("SSN_");
  int nLine=msg.GetNumber("LINE");
  int nColumn=msg.GetNumber("COL_");
  int nLineState=msg.GetNumber("STAT");
  long lElapsedTimeSec=msg.GetNumber("TIME");

  TRACE1(("SendLineState(%d,%d,%d,%d,%d)\n",nSession,nLine,nColumn,nLineState,lElapsedTimeSec));

  if(m_pEvents==0) return false;
  else return m_pEvents->SendLineState(nSession,nLine,nColumn,nLineState,lElapsedTimeSec);
}

bool APDecodeEvents::HandleSendSeizeLine(CMsg2 &msg)
{
  int nSession=msg.GetNumber("SSN_");
  int nLine=msg.GetNumber("LINE");
  int nColumn=msg.GetNumber("COL_");

  TRACE1(("SendSeizeLine(%d,%d,%d)\n",nSession,nLine,nColumn));

  if(m_pEvents==0) return false;
  else return m_pEvents->SendSeizeLine(nSession,nLine,nColumn);
}

bool APDecodeEvents::HandleSendRingLine(CMsg2 &msg)
{
  int nSession=msg.GetNumber("SSN_");
  const char * pszCallerId=msg.GetString("CLID");
  int nLine=msg.GetNumber("LINE");

  TRACE1(("SendRingLine(%d,%s,%d)\n",nSession,pszCallerId==0 ? "(null)" : pszCallerId,nLine));

  if(m_pEvents==0) return false;
  else return m_pEvents->SendRingLine(nSession,pszCallerId,nLine);
}

bool APDecodeEvents::HandleSendDropLine(CMsg2 &msg)
{
  int nSession=msg.GetNumber("SSN_");
  const char * pszCallerId=msg.GetString("CLID");
  int nLine=msg.GetNumber("LINE");

  TRACE1(("SendDropLine(%d,%s,%d)\n",nSession,pszCallerId==0 ? "(null)" : pszCallerId,nLine));

  if(m_pEvents==0) return false;
  else return m_pEvents->SendDropLine(nSession,pszCallerId,nLine);
}

bool APDecodeEvents::HandleSendAnswerLine(CMsg2 &msg)
{
  int nSession=msg.GetNumber("SSN_");
  const char * pszCallerId=msg.GetString("CLID");
  int nLine=msg.GetNumber("LINE");
  int nColumn=msg.GetNumber("COL_");

  TRACE1(("SendAnswerLine(%d,%s,%d,%d)\n",nSession,pszCallerId==0 ? "(null)" : pszCallerId,nLine,nColumn));

  if(m_pEvents==0) return false;
  else return m_pEvents->SendAnswerLine(nSession,pszCallerId,nLine,nColumn);
}

bool APDecodeEvents::HandleSendSelectLine(CMsg2 &msg)
{
  int nSession=msg.GetNumber("SSN_");
  const char * pszCallerId=msg.GetString("CLID");
  int nLine=msg.GetNumber("LINE");
  int nColumn=msg.GetNumber("COL_");

  TRACE1(("SendSelectLine(%d,%s,%d,%d)\n",nSession,pszCallerId==0 ? "(null)" : pszCallerId,nLine,nColumn));

  if(m_pEvents==0) return false;
  else return m_pEvents->SendSelectLine(nSession,pszCallerId,nLine,nColumn);
}

bool APDecodeEvents::HandleSendDialKey(CMsg2 &msg)
{
  int nSession=msg.GetNumber("SSN_");
  char chKey=(char)msg.GetNumber("KEY_");

  TRACE1(("SendDialKey(%d,%c)\n",nSession,chKey));
  
  if(m_pEvents==0) return false;
  else return m_pEvents->SendDialKey(nSession,chKey);
}

bool APDecodeEvents::HandleSendText(CMsg2 &msg)
{
  int nSession=msg.GetNumber("SSN_");
  int nTextType=msg.GetNumber("TTYP");
  const char * pszText=msg.GetString("TEXT");
  int nLine=msg.GetNumber("LINE");

  TRACE1(("SendText(%d,%d,%s,%d)\n",nSession,nTextType,pszText,nLine));

  if(m_pEvents==0) return false;
  else return m_pEvents->SendText(nSession,nTextType,pszText,nLine);
}

bool APDecodeEvents::HandleSendError(CMsg2 &msg)
{
  int nSession=msg.GetNumber("SSN_");
  long nErrorNumber=msg.GetNumber("ENUM");
  const char * pszErrorText=msg.GetString("ETXT");

  TRACE1(("SendError(%d,%d,%s)\n",nSession,nErrorNumber,pszErrorText));

  if(m_pEvents==0) return false;
  else return m_pEvents->SendError(nSession,nErrorNumber,pszErrorText);
}

bool APDecodeEvents::HandleLoginResult(CMsg2 &msg)
{
  bool bResult=msg.GetNumber("RET_")!=0;

  TRACE1(("LoginResult(%s)\n",bResult?"OK":"failed"));

  if(m_pEvents==0) return false;
  else return m_pEvents->LoginResult(bResult);
}

APDecodeEvents::APDecodeEvents(IAPEvents *pEvents)
:
m_pEvents(pEvents)
{
  m_fn[APSRVTOCLI_ENUMERATESHOWSCALLBACK]=&APDecodeEvents::HandleEnumerateShowsCallback;
  m_fn[APSRVTOCLI_ENUMERATEHOSTSCALLBACK]=&APDecodeEvents::HandleEnumerateHostsCallback;
  m_fn[APSRVTOCLI_ENUMERATESHOWSFORHOSTCALLBACK]=&APDecodeEvents::HandleEnumerateShowsForHost;
  m_fn[APSRVTOCLI_ENUMERATEDIRECTORSCALLBACK]=&APDecodeEvents::HandleEnumerateDirectorsCallback;
  m_fn[APSRVTOCLI_SENDSHOWINFO]=&APDecodeEvents::HandleSendShowInfo;
  m_fn[APSRVTOCLI_SENDMODE]=&APDecodeEvents::HandleSendMode;
  m_fn[APSRVTOCLI_SENDSHOWCHANGE]=&APDecodeEvents::HandleSendShowChange;
  m_fn[APSRVTOCLI_SENDRECORD]=&APDecodeEvents::HandleSendRecord;
  m_fn[APSRVTOCLI_SENDBUSYALL]=&APDecodeEvents::HandleSendBusyAll;
  m_fn[APSRVTOCLI_SENDTRANSFER]=&APDecodeEvents::HandleSendTransfer;
  m_fn[APSRVTOCLI_SENDMUTE]=&APDecodeEvents::HandleSendMute;
  m_fn[APSRVTOCLI_SENDRINGMUTE]=&APDecodeEvents::HandleSendRingMute;
  m_fn[APSRVTOCLI_SENDDELAYDUMP]=&APDecodeEvents::HandleSendDelayDump;
  m_fn[APSRVTOCLI_SENDHANDSET]=&APDecodeEvents::HandleSendHandset;
  m_fn[APSRVTOCLI_SENDSPEAKER]=&APDecodeEvents::HandleSendSpeaker;
  m_fn[APSRVTOCLI_SENDLINESTATE]=&APDecodeEvents::HandleSendLineState;
  m_fn[APSRVTOCLI_SENDSEIZELINE]=&APDecodeEvents::HandleSendSeizeLine;
  m_fn[APSRVTOCLI_SENDRINGLINE]=&APDecodeEvents::HandleSendRingLine;
  m_fn[APSRVTOCLI_SENDDROPLINE]=&APDecodeEvents::HandleSendDropLine;
  m_fn[APSRVTOCLI_SENDANSWERLINE]=&APDecodeEvents::HandleSendAnswerLine;
  m_fn[APSRVTOCLI_SENDSELECTLINE]=&APDecodeEvents::HandleSendSelectLine;
  m_fn[APSRVTOCLI_SENDDIALKEY]=&APDecodeEvents::HandleSendDialKey;
  m_fn[APSRVTOCLI_SENDTEXT]=&APDecodeEvents::HandleSendText;
  m_fn[APSRVTOCLI_SENDERROR]=&APDecodeEvents::HandleSendError;
}

// IAPMessageReceiver
bool APDecodeEvents::OnMessage(const char *pData,unsigned cbData)
{
  CMsg2 msg;

  if(msg.SetData(pData,cbData)!=cbData) return false; // not all data was properly parsed!

  long lIndex=msg.GetId();
  if(lIndex==-1) // An index of -1 is returned during initial login
  {
    return HandleLoginResult(msg);
  }

  if(lIndex<0 || lIndex>=APSRVTOCLI_MAX_INDEX) return false; // Message index is not recognized (out of range)

  // Invoke handler
  return (this->*m_fn[lIndex])(msg);
}
